package Kawari::Dictionary;
use utf8;
use Kawari::Script;

sub new{
	my $package = shift;
	$package = ref $package if ref $package;
	my $data = {
		dictionary => {},
		attr => {}
	};
	return bless $data,$package;
}

# TODO: entry name validation
# (temporary dictionary can be satisfied by using temporary instance of this)

sub load{
	my $self = shift;
	my @dict_files = @_;
	for my $dict_file (@dict_files){
		open my $fh,'<:utf8',$dict_file;
		flock $fh,1;
		local $/;
		my $file_string = "\n" . <$fh>;
		close $fh;
		my @mode = ('dict');
		my $sections = [];
		# comments
		$file_string =~ s/(?<=\n)[ \t]*#.*?\n//gs;
		# pragma comments
		$file_string =~ s/(?<=\n):rem\n.*?\n:endrem\n//gs;
		# pragma ignore
		$file_string =~ s/(?<=\n):crypt\n//g;
		$file_string =~ s/(?<=\n):endcrypt\n//g;
		while($file_string !~ /^[ \t\n]*$/){
			$file_string =~ s/^(.*?)(?:(?<=\n)=(kis|dict|end)\n|$)//s;
			my ($section, $next_mode) = ($1,$2);
			push @$sections,{mode => $mode[-1], string => $section};
			defined $next_mode && $next_mode ne 'end' ? push @mode, $next_mode : pop @mode;
		}
		for my $section (@$sections){
			my $string = $section->{string};
			if($section->{mode} eq 'kis'){ # =kis
				my $script = Kawari::Script->new('cmd');
				$script->parse($string.')'); # cmd till )
				$self->push(undef, $script);
			}else{ # =dict
				while($string !~ /^[ \t\n]*$/){
					$string =~ s/^[ \t\n]*(.+?)[ \t]*://s;
					my ($entry_names_str) = $1;
					die 'dict error',"[$1]",$string unless defined $entry_names_str;
					my @entry_names = split /[ \t]*,[ \t]*/,$entry_names_str;
					while($string !~ /^[ \t]*(?:\n|$)/){
						$string =~ s/^[ \t]*,//;
						my $script = Kawari::Script->new();
						$string = $script->parse($string); # parse till , \n and return the rest
						$self->push($_, $script) for @entry_names;
					}
				}
			}
		}
	}
}

sub size{
	my $self = shift;
	my $entry_name = shift;
	return 1 + $#{$self->{dictionary}->{$entry_name}}
}

sub attr{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	my $attr = shift;
	defined $attr ? ($self->{attr}->{$entry_name} = $attr) : return $self->{attr}->{$entry_name};
}

sub set{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	my $value = shift;
	my $index = shift;
	defined $index ? ($self->{dictionary}->{$entry_name}->[$index] = $value) : ($self->{dictionary}->{$entry_name} = [$value]);
}

sub get{
	my $self = shift;
	my $entry_name = shift;
	my $index = shift;
	return defined $index ? $self->{dictionary}->{$entry_name}->[$index] : $self->{dictionary}->{$entry_name};
}

sub push{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	my @value = @_;
	push @{$self->{dictionary}->{$entry_name}},@value;
}

sub unshift{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	my @value = @_;
	unshift @{$self->{dictionary}->{$entry_name}},@value;
}

sub pop{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	return pop @{$self->{dictionary}->{$entry_name}};
}

sub shift{
	my $self = shift;
	my $entry_name = shift;
	return if $self->{attr}->{$entry_name} eq 'readonly';
	return shift @{$self->{dictionary}->{$entry_name}};
}

1;
