package Kawari::Script;
use utf8;

# TODO: execute

my $bra_type = {
	'(' => 'block',
	'$(' => 'cmd',
	'${' => 'call',
	'$[' => 'expr',
	'$' => 'elem',
	"'" => 'squoted',
	'"' => 'dquoted'
};
my $bra_match = qr/\$[({[]|[(\$"']/;
my $cket_match = {
	'plain' => qr/(?=[,\n]|$)/,
	'block' => qr/\)/,
	'cmd' => qr/[;)]/,
	'call' => qr/\}/,
	'expr' => qr/\]/,
	'elem' => qr/\[/,
	'elem_index' => qr/\]/,
	'squoted' => qr/$/,
	'dquoted' => qr/$/
};
my $bracket_string = {
	'plain' => ['',''],
	'block' => ['(',')'],
	'cmd' => ['$(',')'],
	'call' => ['${','}'],
	'expr' => ['$[',']'],
	'elem' => ['$',''],
	'elem_index' => ['[',']'],
	'squoted' => ["",""],
	'dquoted' => ['',''],
	'separator' => [';','']
};

sub new{
	my $package = shift;
	$package = ref $package if ref $package;
	my $type = shift || 'plain';
	my $data = {
		type => $type,
		tree => []
	};
	return bless $data,$package;
}

sub type{shift->{type}}

sub parse{
	my $self = shift;
	my $value = shift;
	# if($self->type eq 'squoted'){
	# 	$value =~ s/\\\\/\\/g;
	# 	$value =~ s/\\'/'/g;
	# }elsif($self->type eq 'dquoted'){
	# 	$value =~ s/\\\\/\\/g;
	# 	$value =~ s/\\"/"/g;
	# }
	if($self->type eq 'squoted' || $self->type eq 'dquoted'){
		$self->_push_value($value);
		return '';
	}
	my $regexp = qr/
		(.*?) # 括弧の前 $1
		( # 括弧の種類判別 $2
		  $bra_match|$cket_match->{$self->type}
		)
		(.*) # 括弧の後 $3
	/xs;# 開き括弧の前後.*を読み取り
	while($value){
		my ($pre_bracket,$bracket,$post_bracket) = $value =~ $regexp;
		if(defined $bracket){ # 括弧がある場合:トークンに括弧前部分/次回括弧判定に括弧後部分
			$self->_push_value($pre_bracket);
			if($bra_type->{$bracket}){
				if($bra_type->{$bracket} eq 'squoted' || $bra_type->{$bracket} eq 'dquoted'){
					$value = $post_bracket;
					$value =~ s/^((?:\\\\|\\$bracket|(?!$bracket).)*?)$bracket//;
					my $script = Kawari::Script->new($bra_type->{$bracket});
					$script->parse($1);
					$self->_push_value($script);
				}else{
					my $script = Kawari::Script->new($bra_type->{$bracket});
					$value = $script->parse($post_bracket);
					$self->_push_value($script);
				}
			}elsif($bracket eq ';'){ # cmd separator
				my $script = Kawari::Script->new('separator');
				$self->_push_value($script);
				$value = $post_bracket;
			}elsif($bracket eq '['){ # elem -> elem_index
				my $script = Kawari::Script->new('elem_index');
				$value = $script->parse($post_bracket);
				$self->_set_index($script);
				return $value;
			}else{
				return $post_bracket;
			}
		}else{
			die "Unmatch bracket.\n",sprintf '%40s',$value;
		}
	}
}

sub _push_value{
	push @{shift->{tree}},shift;
}

sub _set_index{
	my $self = shift;
	my $index = shift;
	$self->{index} = $index;
}

sub execute{}

sub to_string{
	my $self = shift;
	my $string = $bracket_string->{$self->type}->[0];
	for my $elem (@{$self->{tree}}){
		if(ref $elem){
			$string .= $elem->to_string;
		}else{
			$string .= $elem;
		}
	}
	$string .= $bracket_string->{$self->type}->[1];
	if($self->{index}){
		$string .= $self->{index}->to_string;
	}
	if($self->type eq 'plain'){
		$string =~ s/^[ \t]*//;
		$string =~ s/[ \t]*$//;
	}
	return $string;
}

1;
